@@ -14,17 +14,20 @@
  *   Free Software Foundation, Inc.,
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
-#ifdef HAVE_CONFIG_H
 #include "autoconfig.h"
-#endif
 
 #include <stdio.h>
 #include <signal.h>
 #include <errno.h>
 #include <fnmatch.h>
+#ifndef _WIN32
 #include <sys/time.h>
 #include <sys/resource.h>
-#include <fcntl.h>
+#else
+#include <direct.h>
+#endif
+#include "safefcntl.h"
+#include "safeunistd.h"
 
 #include <iostream>
 #include <list>
@@ -37,15 +40,21 @@ using namespace std;
 #include "rclinit.h"
 #include "indexer.h"
 #include "smallut.h"
+#include "chrono.h"
 #include "pathut.h"
+#include "rclutil.h"
 #include "rclmon.h"
 #include "x11mon.h"
 #include "cancelcheck.h"
 #include "rcldb.h"
+#ifndef DISABLE_WEB_INDEXER
 #include "beaglequeue.h"
+#endif
 #include "recollindex.h"
 #include "fsindexer.h"
+#ifndef _WIN32
 #include "rclionice.h"
+#endif
 #include "execmd.h"
 #include "checkretryfailed.h"
 
@@ -82,6 +91,7 @@ static ConfIndexer *confindexer;
 static void cleanup()
 {
     deleteZ(confindexer);
+    recoll_exitready();
 }
 
 // Global stop request flag. This is checked in a number of place in the
@@ -94,41 +104,45 @@ int stopindexing;
 class MyUpdater : public DbIxStatusUpdater {
  public:
     MyUpdater(const RclConfig *config) 
-	: m_prevphase(DbIxStatus::DBIXS_NONE)
-    {
-	m_fd = open(config->getIdxStatusFile().c_str(), 
-		    O_WRONLY|O_CREAT|O_TRUNC, 0600);
-	if (m_fd < 0)
-	    LOGERR(("Can't open/create status file: [%s]\n",
-		    config->getIdxStatusFile().c_str()));
+	: m_file(config->getIdxStatusFile().c_str()),
+	  m_prevphase(DbIxStatus::DBIXS_NONE) {
+        // The total number of files included in the index is actually
+        // difficult to compute from the index itself. For display
+        // purposes, we save it in the status file from indexing to
+        // indexing (mostly...)
+        string stf;
+        if (m_file.get("totfiles", stf)) {
+            status.totfiles = atoi(stf.c_str());
+        }
     }
 
     virtual bool update() 
     {
-	// Update the status file. Avoid doing it too often
-	if (status.phase != m_prevphase || m_chron.millis() > 300) {
+	// Update the status file. Avoid doing it too often. Always do
+	// it at the end (status DONE)
+	if (status.phase == DbIxStatus::DBIXS_DONE || 
+            status.phase != m_prevphase || m_chron.millis() > 300) {
+            if (status.totfiles < status.filesdone) {
+                status.totfiles = status.filesdone;
+            }
 	    m_prevphase = status.phase;
 	    m_chron.restart();
-	    lseek(m_fd, 0, 0);
-	    int fd1 = dup(m_fd);
-	    FILE *fp = fdopen(fd1, "w");
-	    fprintf(fp, "phase = %d\n", int(status.phase));
-	    fprintf(fp, "docsdone = %d\n", status.docsdone);
-	    fprintf(fp, "filesdone = %d\n", status.filesdone);
-	    fprintf(fp, "dbtotdocs = %d\n", status.dbtotdocs);
-	    fprintf(fp, "fn = %s\n", status.fn.c_str());
-	    if (ftruncate(m_fd, off_t(ftell(fp))) < 0) {
-		// ? kill compiler warning about ignoring ftruncate return
-		LOGDEB(("Status update: ftruncate failed\n"));
-	    }
-            // Flush data and closes fd1. m_fd still valid
-	    fclose(fp); 
+            m_file.holdWrites(true);
+            m_file.set("phase", int(status.phase));
+	    m_file.set("docsdone", status.docsdone);
+	    m_file.set("filesdone", status.filesdone);
+	    m_file.set("fileerrors", status.fileerrors);
+	    m_file.set("dbtotdocs", status.dbtotdocs);
+	    m_file.set("totfiles", status.totfiles);
+	    m_file.set("fn", status.fn);
+            m_file.holdWrites(false);
 	}
 
 	if (stopindexing) {
 	    return false;
 	}
 
+#ifndef DISABLE_X11MON
 	// If we are in the monitor, we also need to check X11 status
 	// during the initial indexing pass (else the user could log
 	// out and the indexing would go on, not good (ie: if the user
@@ -138,12 +152,12 @@ class MyUpdater : public DbIxStatusUpdater {
 	    stopindexing = true;
 	    return false;
 	}
-
+#endif
 	return true;
     }
 
 private:
-    int    m_fd;
+    ConfSimple m_file;
     Chrono m_chron;
     DbIxStatus::Phase m_prevphase;
 };
@@ -172,11 +186,13 @@ static void makeIndexerOrExit(RclConfig *config, bool inPlaceReset)
 
 void rclIxIonice(const RclConfig *config)
 {
+#ifndef _WIN32
     string clss, classdata;
     if (!config->getConfParam("monioniceclass", clss) || clss.empty())
 	clss = "3";
     config->getConfParam("monioniceclassdata", classdata);
     rclionice(clss, classdata);
+#endif
 }
 
 class MakeListWalkerCB : public FsTreeWalkerCB {
@@ -291,7 +307,7 @@ static bool checktopdirs(RclConfig *config, vector<string>& nonexist)
 
     for (vector<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
 	*it = path_tildexpand(*it);
-        if (!it->size() || (*it)[0] != '/') {
+        if (!it->size() || !path_isabsolute(*it)) {
             if ((*it)[0] == '~') {
                 cerr << "Tilde expansion failed: " << *it << endl;
                 LOGERR(("recollindex: tilde expansion failed: %s\n",
@@ -303,30 +319,13 @@ static bool checktopdirs(RclConfig *config, vector<string>& nonexist)
             }
             return false;
         }
-        if (access(it->c_str(), 0) < 0) {
+        if (!path_exists(*it)) {
             nonexist.push_back(*it);
         }
     }
 
-    // Check skippedPaths too, but only the user part (shallow==true), not
-    // the default values (e.g. /media, which might not exist).
-    if (config->getConfParam("skippedPaths", &tdl, true)) {
-        for (vector<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
-            *it = path_tildexpand(*it);
-            if (access(it->c_str(), 0) < 0) {
-                nonexist.push_back(*it);
-            }
-        }
-    }
+    // We'd like to check skippedPaths too, but these are wildcard exprs, so reasonably can't
 
-    if (config->getConfParam("daemSkippedPaths", &tdl, true)) {
-        for (vector<string>::iterator it = tdl.begin(); it != tdl.end(); it++) {
-            *it = path_tildexpand(*it);
-            if (access(it->c_str(), 0) < 0) {
-                nonexist.push_back(*it);
-            }
-        }
-    }
     return true;
 }
 
@@ -415,8 +414,10 @@ int main(int argc, char **argv)
     // The reexec struct is used by the daemon to shed memory after
     // the initial indexing pass and to restart when the configuration
     // changes
+#ifndef _WIN32
     o_reexec = new ReExec;
     o_reexec->init(argc, argv);
+#endif
 
     thisprog = argv[0];
     argc--; argv++;
@@ -483,13 +484,15 @@ int main(int argc, char **argv)
     }
     string reason;
     RclInitFlags flags = (op_flags & OPT_m) && !(op_flags&OPT_D) ? 
-	RCLINIT_DAEMON : RCLINIT_NONE;
+	RCLINIT_DAEMON : RCLINIT_IDX;
     config = recollinit(flags, cleanup, sigcleanup, reason, &a_config);
     if (config == 0 || !config->ok()) {
 	cerr << "Configuration problem: " << reason << endl;
 	exit(1);
     }
+#ifndef _WIN32
     o_reexec->atexit(cleanup);
+#endif
 
     vector<string> nonexist;
     if (!checktopdirs(config, nonexist))
@@ -548,12 +551,13 @@ int main(int argc, char **argv)
     // Log something at LOGINFO to reset the trace file. Else at level
     // 3 it's not even truncated if all docs are up to date.
     LOGINFO(("recollindex: starting up\n"));
-
+#ifndef _WIN32
     if (setpriority(PRIO_PROCESS, 0, 20) != 0) {
         LOGINFO(("recollindex: can't setpriority(), errno %d\n", errno));
     }
     // Try to ionice. This does not work on all platforms
     rclIxIonice(config);
+#endif
 
     if (op_flags & OPT_r) {
 	if (argc != 1) 
@@ -622,15 +626,17 @@ int main(int argc, char **argv)
 	lockorexit(&pidfile);
 	if (!(op_flags&OPT_D)) {
 	    LOGDEB(("recollindex: daemonizing\n"));
+#ifndef _WIN32
 	    if (daemon(0,0) != 0) {
 	      fprintf(stderr, "daemon() failed, errno %d\n", errno);
 	      LOGERR(("daemon() failed, errno %d\n", errno));
 	      exit(1);
 	    }
+#endif
 	}
 	// Need to rewrite pid, it changed
 	pidfile.write_pid();
-
+#ifndef _WIN32
         // Not too sure if I have to redo the nice thing after daemon(),
         // can't hurt anyway (easier than testing on all platforms...)
         if (setpriority(PRIO_PROCESS, 0, 20) != 0) {
@@ -638,6 +644,7 @@ int main(int argc, char **argv)
         }
 	// Try to ionice. This does not work on all platforms
 	rclIxIonice(config);
+#endif
 
 	if (sleepsecs > 0) {
 	    LOGDEB(("recollindex: sleeping %d\n", sleepsecs));
@@ -665,6 +672,7 @@ int main(int argc, char **argv)
                 }
             }
 	    deleteZ(confindexer);
+#ifndef _WIN32
 	    o_reexec->insertArgs(vector<string>(1, "-n"));
 	    LOGINFO(("recollindex: reexecuting with -n after initial full pass\n"));
 	    // Note that -n will be inside the reexec when we come
@@ -672,6 +680,7 @@ int main(int argc, char **argv)
 	    // starting a config change exec to ensure that we do a
 	    // purging pass in this case.
 	    o_reexec->reexec();
+#endif
 	}
         if (updater) {
 	    updater->status.phase = DbIxStatus::DBIXS_MONITOR;
@@ -698,7 +707,6 @@ int main(int argc, char **argv)
 	makeIndexerOrExit(config, inPlaceReset);
 	bool status = confindexer->index(rezero, ConfIndexer::IxTAll, 
                                          indexerFlags);
-
         // Record success of indexing pass with failed files retries.
         if (status && !(indexerFlags & ConfIndexer::IxFNoRetryFailed)) {
             checkRetryFailed(config, true);
