@@ -19,11 +19,11 @@
 #include "autoconfig.h"
 
 #include <stdio.h>
-#include <unistd.h>
-#include <fcntl.h>
-#include <sys/types.h>
-#include <sys/stat.h>
 #include <errno.h>
+#include "safefcntl.h"
+#include <sys/types.h>
+#include "safesysstat.h"
+#include "safeunistd.h"
 
 #include <string>
 #include <iostream>
@@ -175,7 +175,7 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
             l_mime = *imime;
     }
 
-    size_t docsize = stp->st_size;
+    off_t docsize = stp->st_size;
 
     if (!l_mime.empty()) {
 	// Has mime: check for a compressed file. If so, create a
@@ -195,7 +195,7 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
 		m_fn = m_tfile;
 		// Stat the uncompressed file, mainly to get the size
 		struct stat ucstat;
-		if (stat(m_fn.c_str(), &ucstat) != 0) {
+		if (path_fileprops(m_fn, &ucstat) != 0) {
 		    LOGERR(("FileInterner: can't stat the uncompressed file"
 			    "[%s] errno %d\n", m_fn.c_str(), errno));
 		    return;
@@ -222,7 +222,7 @@ void FileInterner::init(const string &f, const struct stat *stp, RclConfig *cnf,
     m_mimetype = l_mime;
     RecollFilter *df = getMimeHandler(l_mime, m_cfg, !m_forPreview);
 
-    if (!df or df->is_unknown()) {
+    if (!df || df->is_unknown()) {
 	// No real handler for this type, for now :( 
 	LOGDEB(("FileInterner:: unprocessed mime: [%s] [%s]\n", 
 		l_mime.c_str(), f.c_str()));
@@ -293,7 +293,7 @@ void FileInterner::init(const string &data, RclConfig *cnf,
 	result = df->set_document_data(m_mimetype, data.c_str(), data.length());
     } else if (df->is_data_input_ok(Dijon::Filter::DOCUMENT_FILE_NAME)) {
 	TempFile temp = dataToTempFile(data, m_mimetype);
-	if (temp.isNotNull() && 
+	if (temp && 
 	    (result = df->set_document_file(m_mimetype, temp->filename()))) {
 	    m_tmpflgs[m_handlers.size()] = true;
 	    m_tempfiles.push_back(temp);
@@ -635,7 +635,7 @@ void FileInterner::popHandler()
 {
     if (m_handlers.empty())
 	return;
-    int i = m_handlers.size() - 1;
+    size_t i = m_handlers.size() - 1;
     if (m_tmpflgs[i]) {
 	m_tempfiles.pop_back();
 	m_tmpflgs[i] = false;
@@ -707,7 +707,7 @@ int FileInterner::addHandler()
 	setres = newflt->set_document_data(mimetype,txt->c_str(),txt->length());
     } else if (newflt->is_data_input_ok(Dijon::Filter::DOCUMENT_FILE_NAME)) {
 	TempFile temp = dataToTempFile(*txt, mimetype);
-	if (temp.isNotNull() && 
+	if (temp && 
 	    (setres = newflt->set_document_file(mimetype, temp->filename()))) {
 	    m_tmpflgs[m_handlers.size()] = true;
 	    m_tempfiles.push_back(temp);
@@ -749,7 +749,7 @@ FileInterner::Status FileInterner::internfile(Rcl::Doc& doc, const string& ipath
     LOGDEB(("FileInterner::internfile. ipath [%s]\n", ipath.c_str()));
 
     // Get rid of possible image tempfile from older call
-    m_imgtmp.release();
+    m_imgtmp.reset();
 
     if (m_handlers.size() < 1) {
 	// Just means the constructor failed
@@ -763,7 +763,6 @@ FileInterner::Status FileInterner::internfile(Rcl::Doc& doc, const string& ipath
     // We set the ipath for the first handler here, others are set
     // when they're pushed on the stack
     vector<string> vipath;
-    int vipathidx = 0;
     if (!ipath.empty()) {
 	vector<string> lipath;
 	stringToTokens(ipath, lipath, cstr_isep, true);
@@ -1043,7 +1042,7 @@ bool FileInterner::isCompressed(const string& fn, RclConfig *cnf)
 {
     LOGDEB(("FileInterner::isCompressed: [%s]\n", fn.c_str()));
     struct stat st;
-    if (stat(fn.c_str(), &st) < 0) {
+    if (path_fileprops(fn, &st) < 0) {
         LOGERR(("FileInterner::isCompressed: can't stat [%s]\n", fn.c_str()));
         return false;
     }
@@ -1067,7 +1066,7 @@ bool FileInterner::maybeUncompressToTemp(TempFile& temp, const string& fn,
 {
     LOGDEB(("FileInterner::maybeUncompressToTemp: [%s]\n", fn.c_str()));
     struct stat st;
-    if (stat(fn.c_str(), &st) < 0) {
+    if (path_fileprops(fn.c_str(), &st) < 0) {
         LOGERR(("FileInterner::maybeUncompressToTemp: can't stat [%s]\n", 
                 fn.c_str()));
         return false;
@@ -1123,7 +1122,7 @@ bool FileInterner::maybeUncompressToTemp(TempFile& temp, const string& fn,
 #include <stdlib.h>
 #include <iostream>
 #include <string>
-#include <sys/stat.h>
+#include "safesysstat.h"
 
 using namespace std;
 
