@@ -18,13 +18,20 @@
 #include "autoconfig.h"
 
 #include <stdio.h>
-#include <fcntl.h>
 #include <errno.h>
-#include <unistd.h>
+#include <sys/types.h>
+#include "safesysstat.h"
 #include <time.h>
+
+#if defined(_WIN32)
+#define USING_STD_REGEX
+#endif
+
+#ifdef USING_STD_REGEX
+#include <regex>
+#else
 #include <regex.h>
-#include <sys/stat.h>
-#include <errno.h>
+#endif
 
 #include <cstring>
 #include <map>
@@ -72,6 +79,14 @@ static PTMutexInit o_mcache_mutex;
  * offsets for all message "From_" lines follow. The format is purely
  * binary, values are not even byte-swapped to be proc-idependant.
  */
+
+#ifdef _WIN32
+// vc++ does not let define an array of size o_b1size because non-const??
+#define M_o_b1size 1024
+#else
+#define M_o_b1size o_b1size
+#endif
+
 class MboxCache {
 public:
     typedef MimeHandlerMbox::mbhoff_type mbhoff_type;
@@ -100,7 +115,7 @@ public:
         }
         FpKeeper keeper(&fp);
 
-        char blk1[o_b1size];
+        char blk1[M_o_b1size];
         if (fread(blk1, 1, o_b1size, fp) != o_b1size) {
             LOGDEB0(("MboxCache::get_offsets: read blk1 errno %d\n", errno));
             return -1;
@@ -113,19 +128,19 @@ public:
             return -1;
         }
         if (fseeko(fp, cacheoffset(msgnum), SEEK_SET) != 0) {
-            LOGDEB0(("MboxCache::get_offsets: seek %lld errno %d\n",
-                     cacheoffset(msgnum), errno));
+            LOGDEB0(("MboxCache::get_offsets: seek %s errno %d\n",
+                     lltodecstr(cacheoffset(msgnum)).c_str(), errno));
             return -1;
         }
         mbhoff_type offset = -1;
-        int ret;
+        size_t ret;
         if ((ret = fread(&offset, 1, sizeof(mbhoff_type), fp))
             != sizeof(mbhoff_type)) {
             LOGDEB0(("MboxCache::get_offsets: read ret %d errno %d\n", 
                      ret, errno));
             return -1;
         }
-        LOGDEB0(("MboxCache::get_offsets: ret %lld\n", (long long)offset));
+        LOGDEB0(("MboxCache::get_offsets: ret %s\n", lltodecstr(offset).c_str()));
         return offset;
     }
 
@@ -181,13 +196,7 @@ public:
             }
             m_minfsize = minmbs * 1000 * 1000;
 
-            config->getConfParam("mboxcachedir", m_dir);
-            if (m_dir.empty())
-                m_dir = "mboxcache";
-            m_dir = path_tildexpand(m_dir);
-            // If not an absolute path, compute relative to config dir
-            if (m_dir.at(0) != '/')
-                m_dir = path_cat(config->getConfDir(), m_dir);
+            m_dir = config->getMboxcacheDir();
             m_ok = true;
         }
         return m_ok;
@@ -228,7 +237,6 @@ private:
 };
 
 const size_t MboxCache::o_b1size = 1024;
-
 static class MboxCache o_mcache;
 
 static const string cstr_keyquirks("mhmboxquirks");
@@ -272,9 +280,15 @@ bool MimeHandlerMbox::set_document_file(const string& mt, const string &fn)
         // perror("fcntl");
     }
 #endif
-    fseek((FILE *)m_vfp, 0, SEEK_END);
-    m_fsize = ftell((FILE*)m_vfp);
-    fseek((FILE*)m_vfp, 0, SEEK_SET);
+    // Used to use ftell() here: no good beyond 2GB
+    {struct stat st;
+	if (fstat(fileno((FILE*)m_vfp), &st) < 0) {
+	    LOGERR(("MimeHandlerMbox:setdocfile: fstat(%s) failed errno %d\n",
+		    fn.c_str(), errno));
+	    return false;
+	}
+	m_fsize = st.st_size;
+    }
     m_havedoc = true;
     m_offsets.clear();
     m_quirks = 0;
@@ -290,7 +304,7 @@ bool MimeHandlerMbox::set_document_file(const string& mt, const string &fn)
 
     // And double check for thunderbird 
     string tbirdmsf = fn + ".msf";
-    if ((m_quirks&MBOXQUIRK_TBIRD) == 0 && access(tbirdmsf.c_str(), 0) == 0) {
+    if ((m_quirks&MBOXQUIRK_TBIRD) == 0 && path_exists(tbirdmsf)) {
 	LOGDEB(("MimeHandlerMbox: detected unconfigured tbird mbox in %s\n",
 		fn.c_str()));
 	m_quirks |= MBOXQUIRK_TBIRD;
@@ -303,7 +317,7 @@ bool MimeHandlerMbox::set_document_file(const string& mt, const string &fn)
 typedef char line_type[LL+10];
 static inline void stripendnl(line_type& line, int& ll)
 {
-    ll = strlen(line);
+    ll = int(strlen(line));
     while (ll > 0) {
 	if (line[ll-1] == '\n' || line[ll-1] == '\r') {
 	    line[ll-1] = 0;
@@ -372,9 +386,19 @@ static const  char *frompat =
 // exactly like: From ^M (From followed by space and eol). We only
 // test for this if QUIRKS_TBIRD is set
 static const char *miniTbirdFrom = "^From $";
-
+#ifndef USING_STD_REGEX
 static regex_t fromregex;
 static regex_t minifromregex;
+#define M_regexec(A,B,C,D,E) regexec(&(A),B,C,D,E)
+#else
+basic_regex<char> fromregex;
+basic_regex<char> minifromregex;
+#define REG_NOSUB std::regex_constants::nosubs
+#define REG_EXTENDED std::regex_constants::extended
+#define M_regexec(A, B, C, D, E) (!regex_match(B,A))
+
+#endif
+
 static bool regcompiled;
 static PTMutexInit o_regex_mutex;
 
@@ -386,8 +410,13 @@ static void compileregexes()
     // that we are alone.
     if (regcompiled)
 	return;
+#ifndef USING_STD_REGEX
     regcomp(&fromregex, frompat, REG_NOSUB|REG_EXTENDED);
     regcomp(&minifromregex, miniTbirdFrom, REG_NOSUB|REG_EXTENDED);
+#else
+    fromregex = basic_regex<char>(frompat, REG_NOSUB | REG_EXTENDED);
+    minifromregex = basic_regex<char>(miniTbirdFrom, REG_NOSUB | REG_EXTENDED);
+#endif
     regcompiled = true;
 }
 
@@ -436,9 +465,9 @@ bool MimeHandlerMbox::next_document()
             (off = o_mcache.get_offset(m_config, m_udi, mtarg)) >= 0 && 
             fseeko(fp, (off_t)off, SEEK_SET) >= 0 && 
             fgets(line, LL, fp) &&
-            (!regexec(&fromregex, line, 0, 0, 0) || 
+            (!M_regexec(fromregex, line, 0, 0, 0) || 
 	     ((m_quirks & MBOXQUIRK_TBIRD) && 
-	      !regexec(&minifromregex, line, 0, 0, 0)))	) {
+	      !M_regexec(minifromregex, line, 0, 0, 0)))	) {
                 LOGDEB0(("MimeHandlerMbox: Cache: From_ Ok\n"));
                 fseeko(fp, (off_t)off, SEEK_SET);
                 m_msgnum = mtarg -1;
@@ -481,11 +510,11 @@ bool MimeHandlerMbox::next_document()
 		/* The 'F' compare is redundant but it improves performance
 		   A LOT */
 		if (line[0] == 'F' && (
-		    !regexec(&fromregex, line, 0, 0, 0) || 
+		    !M_regexec(fromregex, line, 0, 0, 0) || 
 		    ((m_quirks & MBOXQUIRK_TBIRD) && 
-		     !regexec(&minifromregex, line, 0, 0, 0)))
+		     !M_regexec(minifromregex, line, 0, 0, 0)))
 		    ) {
-		    LOGDEB1(("MimeHandlerMbox: msgnum %d, "
+		    LOGDEB0(("MimeHandlerMbox: msgnum %d, "
 		     "From_ at line %d: [%s]\n", m_msgnum, m_lineno, line));
 		    if (storeoffsets)
 			m_offsets.push_back(message_end);
