@@ -18,21 +18,17 @@
  */
 
 #include <string>
-using std::string;
 
-#include <qglobal.h>
-#include <qstring.h>
+#include <QString>
+#include <QUrl>
 
-#include <kurl.h>
-#include <kio/global.h>
 #include <kio/slavebase.h>
-#include <kdeversion.h>
 
 #include "rclconfig.h"
 #include "rcldb.h"
-#include "reslistpager.h"
 #include "docseq.h"
-#include "refcntr.h"
+#include "reslistpager.h"
+#include MEMORY_INCLUDE
 
 class RecollProtocol;
 
@@ -40,16 +36,19 @@ class RecollProtocol;
 class RecollKioPager : public ResListPager {
 public:
     RecollKioPager() : m_parent(0) {}
-    void setParent(RecollProtocol *proto) {m_parent = proto;}
+    void setParent(RecollProtocol *proto) {
+        m_parent = proto;
+    }
 
-    virtual bool append(const string& data);
-    virtual bool append(const string& data, int, const Rcl::Doc&)
-    {return append(data);}
-    virtual string detailsLink();
-    virtual const string &parFormat();
-    virtual string nextUrl();
-    virtual string prevUrl();
-    virtual string pageTop();
+    virtual bool append(const std::string& data);
+    virtual bool append(const std::string& data, int, const Rcl::Doc&) {
+        return append(data);
+    }
+    virtual std::string detailsLink();
+    virtual const std::string& parFormat();
+    virtual std::string nextUrl();
+    virtual std::string prevUrl();
+    virtual std::string pageTop();
 
 private:
     RecollProtocol *m_parent;
@@ -63,7 +62,7 @@ public:
     int page;
     bool isDetReq;
     bool sameQuery(const QueryDesc& o) const {
-	return !opt.compare(o.opt) && !query.compare(o.query);
+        return !opt.compare(o.opt) && !query.compare(o.query);
     }
 };
 
@@ -71,32 +70,44 @@ public:
 // and tell what we should do with it
 class UrlIngester {
 public:
-    UrlIngester(RecollProtocol *p, const KUrl& url);
+    UrlIngester(RecollProtocol *p, const QUrl& url);
     enum RootEntryType {UIRET_NONE, UIRET_ROOT, UIRET_HELP, UIRET_SEARCH};
     bool isRootEntry(RootEntryType *tp) {
-	if (m_type != UIMT_ROOTENTRY) return false;
-	*tp = m_retType;
-	return true;
+        if (m_type != UIMT_ROOTENTRY) {
+            return false;
+        }
+        *tp = m_retType;
+        return true;
     }
     bool isQuery(QueryDesc *q) {
-	if (m_type != UIMT_QUERY) return false;
-	*q = m_query;
-	return true;
+        if (m_type != UIMT_QUERY) {
+            return false;
+        }
+        *q = m_query;
+        return true;
     }
     bool isResult(QueryDesc *q, int *num) {
-	if (m_type != UIMT_QUERYRESULT) return false;
-	*q = m_query;
-	*num = m_resnum;
-	return true;
+        if (m_type != UIMT_QUERYRESULT) {
+            return false;
+        }
+        *q = m_query;
+        *num = m_resnum;
+        return true;
     }
     bool isPreview(QueryDesc *q, int *num) {
-	if (m_type != UIMT_PREVIEW) return false;
-	*q = m_query;
-	*num = m_resnum;
-	return true;
+        if (m_type != UIMT_PREVIEW) {
+            return false;
+        }
+        *q = m_query;
+        *num = m_resnum;
+        return true;
+    }
+    bool endSlashQuery() {
+        return m_slashend;
+    }
+    bool alwaysDir() {
+        return m_alwaysdir;
     }
-    bool endSlashQuery() {return m_slashend;}
-    bool alwaysDir() {return m_alwaysdir;}
 
 private:
     RecollProtocol *m_parent;
@@ -106,18 +117,18 @@ private:
     RootEntryType   m_retType;
     int             m_resnum;
     enum MyType {UIMT_NONE, UIMT_ROOTENTRY, UIMT_QUERY, UIMT_QUERYRESULT,
-		 UIMT_PREVIEW};
+                 UIMT_PREVIEW
+                };
     MyType           m_type;
 };
 
-
 /**
  * A KIO slave to execute and display Recoll searches.
  *
  * Things are made a little complicated because KIO slaves can't hope
  * that their internal state will remain consistent with their user
  * application state: slaves die, are restarted, reused, at random
- * between requests. 
+ * between requests.
  * In our case, this means that any request has to be processed
  * without reference to the last operation performed. Ie, if the
  * search parameters are not those from the last request, the search
@@ -134,44 +145,43 @@ private:
  * functionality, and one based on a directory listing model, which
  * will always be more limited, but may be useful in some cases to
  * allow easy copying of files etc. Which one is in use is decided by
- * the form of the URL. 
+ * the form of the URL.
  */
 class RecollProtocol : public KIO::SlaveBase {
- public:
-    RecollProtocol(const QByteArray &pool, const QByteArray &app );
+public:
+    RecollProtocol(const QByteArray& pool, const QByteArray& app);
     virtual ~RecollProtocol();
-    virtual void mimetype(const KUrl& url);
-    virtual void get(const KUrl& url);
+    virtual void mimetype(const QUrl& url);
+    virtual void get(const QUrl& url);
     // The directory mode is not available with KDE 4.0, I could find
     // no way to avoid crashing kdirmodel
-#if KDE_IS_VERSION(4,1,0)
-    virtual void stat(const KUrl & url);
-    virtual void listDir(const KUrl& url);
-#endif
+    virtual void stat(const QUrl& url);
+    virtual void listDir(const QUrl& url);
 
     static RclConfig  *o_rclconfig;
 
     friend class RecollKioPager;
     friend class UrlIngester;
 
- private:
-    bool maybeOpenDb(string& reason);
-    bool URLToQuery(const KUrl &url, QString& q, QString& opt, int *page=0);
+private:
+    bool maybeOpenDb(std::string& reason);
+    bool URLToQuery(const QUrl& url, QString& q, QString& opt, int *page = 0);
     bool doSearch(const QueryDesc& qd);
 
     void searchPage();
     void queryDetails();
-    string makeQueryUrl(int page, bool isdet = false);
+    std::string makeQueryUrl(int page, bool isdet = false);
     bool syncSearch(const QueryDesc& qd);
     void htmlDoSearch(const QueryDesc& qd);
     void showPreview(const Rcl::Doc& doc);
-    bool isRecollResult(const KUrl &url, int *num, QString* q);
+    bool isRecollResult(const QUrl& url, int *num, QString* q);
 
     bool        m_initok;
     Rcl::Db    *m_rcldb;
-    string      m_reason;
+    std::string      m_reason;
     bool        m_alwaysdir;
-    string      m_stemlang; // english by default else env[RECOLL_KIO_STEMLANG]
+    // english by default else env[RECOLL_KIO_STEMLANG]
+    std::string      m_stemlang;
 
     // Search state: because of how the KIO slaves are used / reused,
     // we can't be sure that the next request will be for the same
@@ -181,11 +191,20 @@ class RecollProtocol : public KIO::SlaveBase {
     // (one slave several konqueror windows) would be to have a small
     // cache of recent searches kept open.
     RecollKioPager m_pager;
-    RefCntr<DocSequence> m_source;
+    STD_SHARED_PTR<DocSequence> m_source;
     // Note: page here is not used, current page always comes from m_pager.
     QueryDesc      m_query;
 };
 
-extern "C" {int kdemain(int, char**);}
+extern "C" {
+    __attribute__((visibility("default"))) int
+    kdemain(int argc, char **argv);
+}
+
+inline QString u8s2qs(const string& s)
+{
+    return QString::fromUtf8(s.c_str());
+}
+
 
 #endif // _RECOLL_H
