@@ -14,6 +14,8 @@
  *   Free Software Foundation, Inc.,
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
+#include "autoconfig.h"
+
 #include "advsearch_w.h"
 
 #include <qvariant.h>
@@ -50,7 +52,7 @@ void AdvSearch::init()
 {
     (void)new HelpClient(this);
     HelpClient::installMap((const char *)objectName().toUtf8(), 
-			   "RCL.SEARCH.COMPLEX");
+			   "RCL.SEARCH.GUI.COMPLEX");
 
     // signals and slots connections
     connect(delFiltypPB, SIGNAL(clicked()), this, SLOT(delFiltypPB_clicked()));
@@ -89,13 +91,13 @@ void AdvSearch::init()
     // Tune initial state according to last saved
     {
 	vector<SearchClauseW *>::iterator cit = m_clauseWins.begin();
-	for (vector<int>::iterator it = prefs.advSearchClauses.begin(); 
-	     it != prefs.advSearchClauses.end(); it++) {
-	    if (cit != m_clauseWins.end()) {
-		(*cit)->tpChange(*it);
+        unsigned int existing = m_clauseWins.size();
+	for (unsigned int i = 0; i < prefs.advSearchClauses.size(); i++) {
+	    if (i < existing) {
+		(*cit)->tpChange(prefs.advSearchClauses[i]);
 		cit++;
 	    } else {
-		addClause(*it);
+		addClause(prefs.advSearchClauses[i]);
 	    }
 	}
     }
@@ -162,12 +164,6 @@ void AdvSearch::saveCnf()
     }
 }
 
-bool AdvSearch::close()
-{
-    saveCnf();
-    return QWidget::close();
-}
-
 void AdvSearch::addClause()
 {
     addClause(0);
@@ -364,7 +360,7 @@ using namespace Rcl;
 void AdvSearch::runSearch()
 {
     string stemLang = prefs.stemlang();
-    RefCntr<SearchData> sdata(new SearchData(conjunctCMB->currentIndex() == 0 ?
+    STD_SHARED_PTR<SearchData> sdata(new SearchData(conjunctCMB->currentIndex() == 0 ?
 					     SCLT_AND : SCLT_OR, stemLang));
     bool hasclause = false;
 
@@ -424,19 +420,20 @@ void AdvSearch::runSearch()
 
     if (!subtreeCMB->currentText().isEmpty()) {
 	QString current = subtreeCMB->currentText();
+
         Rcl::SearchDataClausePath *pathclause = 
             new Rcl::SearchDataClausePath((const char*)current.toLocal8Bit(), 
                                           direxclCB->isChecked());
         if (sdata->getTp() == SCLT_AND) {
             sdata->addClause(pathclause);
         } else {
-            RefCntr<SearchData> nsdata(new SearchData(SCLT_AND, stemLang));
+            STD_SHARED_PTR<SearchData> 
+                nsdata(new SearchData(SCLT_AND, stemLang));
             nsdata->addClause(new Rcl::SearchDataClauseSub(sdata));
             nsdata->addClause(pathclause);
             sdata = nsdata;
         }
 
-
 	// Keep history clean and sorted. Maybe there would be a
 	// simpler way to do this
 	list<QString> entries;
@@ -457,7 +454,6 @@ void AdvSearch::runSearch()
 	for (int index = 0; index < subtreeCMB->count(); index++)
 	    prefs.asearchSubdirHist.push_back(subtreeCMB->itemText(index));
     }
-
     saveCnf();
     g_advshistory && g_advshistory->push(sdata);
     emit startSearch(sdata, false);
@@ -466,7 +462,7 @@ void AdvSearch::runSearch()
 
 // Set up fields from existing search data, which must be compatible
 // with what we can do...
-void AdvSearch::fromSearch(RefCntr<SearchData> sdata)
+void AdvSearch::fromSearch(STD_SHARED_PTR<SearchData> sdata)
 {
     if (sdata->m_tp == SCLT_OR)
 	conjunctCMB->setCurrentIndex(1);
@@ -564,8 +560,8 @@ void AdvSearch::slotHistoryNext()
 {
     if (g_advshistory == 0)
 	return;
-    RefCntr<Rcl::SearchData> sd = g_advshistory->getnewer();
-    if (sd.isNull())
+    STD_SHARED_PTR<Rcl::SearchData> sd = g_advshistory->getnewer();
+    if (!sd)
 	return;
     fromSearch(sd);
 }
@@ -574,8 +570,8 @@ void AdvSearch::slotHistoryPrev()
 {
     if (g_advshistory == 0)
 	return;
-    RefCntr<Rcl::SearchData> sd = g_advshistory->getolder();
-    if (sd.isNull())
+    STD_SHARED_PTR<Rcl::SearchData> sd = g_advshistory->getolder();
+    if (!sd)
 	return;
     fromSearch(sd);
 }
