@@ -16,6 +16,9 @@
  */
 #include "autoconfig.h"
 
+#include <signal.h>
+#include "safeunistd.h"
+
 #include <QMessageBox>
 #include <QTimer>
 
@@ -28,7 +31,7 @@
 
 using namespace std;
 
-void RclMain::idxStatus()
+void RclMain::updateIdxStatus()
 {
     ConfSimple cs(theconfig->getIdxStatusFile().c_str(), 1);
     QString msg = tr("Indexing in progress: ");
@@ -37,12 +40,11 @@ void RclMain::idxStatus()
     cs.get("phase", val);
     status.phase = DbIxStatus::Phase(atoi(val.c_str()));
     cs.get("fn", status.fn);
-    cs.get("docsdone", val);
-    status.docsdone = atoi(val.c_str());
-    cs.get("filesdone", val);
-    status.filesdone = atoi(val.c_str());
-    cs.get("dbtotdocs", val);
-    status.dbtotdocs = atoi(val.c_str());
+    cs.get("docsdone", &status.docsdone);
+    cs.get("filesdone", &status.filesdone);
+    cs.get("fileerrors", &status.fileerrors);
+    cs.get("dbtotdocs", &status.dbtotdocs);
+    cs.get("totfiles", &status.totfiles);
 
     QString phs;
     switch (status.phase) {
@@ -58,11 +60,14 @@ void RclMain::idxStatus()
     msg += phs + " ";
     if (status.phase == DbIxStatus::DBIXS_FILES) {
 	char cnts[100];
-	if (status.dbtotdocs > 0)
-	    sprintf(cnts,"(%d/%d/%d) ", status.docsdone, status.filesdone, 
-		    status.dbtotdocs);
-	else
-	    sprintf(cnts, "(%d/%d) ", status.docsdone, status.filesdone);
+	if (status.dbtotdocs > 0) {
+	    sprintf(cnts, "(%d docs/%d files/%d errors/%d tot files) ",
+                    status.docsdone, status.filesdone, status.fileerrors,
+		    status.totfiles);
+        } else {
+	    sprintf(cnts, "(%d docs/%d files/%d errors) ",
+                    status.docsdone, status.filesdone, status.fileerrors);
+        }
 	msg += QString::fromUtf8(cnts) + " ";
     }
     string mf;int ecnt = 0;
@@ -105,7 +110,7 @@ void RclMain::periodic100()
 	    // update/show status even if the status file did not
 	    // change (else the status line goes blank during
 	    // lengthy operations).
-	    idxStatus();
+	    updateIdxStatus();
 	}
     }
     // Update the "start/stop indexing" menu entry, can't be done from
@@ -120,7 +125,16 @@ void RclMain::periodic100()
 	periodictimer->setInterval(200);
     } else {
 	Pidfile pidfile(theconfig->getPidfile());
-	if (pidfile.open() == 0) {
+        pid_t pid = pidfile.open();
+        if (pid == getpid()) {
+            // Locked by me
+	    m_indexerState = IXST_NOTRUNNING;
+	    fileToggleIndexingAction->setText(tr("Index locked"));
+	    fileToggleIndexingAction->setEnabled(false);
+	    fileRebuildIndexAction->setEnabled(false);
+            actionSpecial_Indexing->setEnabled(false);
+	    periodictimer->setInterval(1000);
+        } else if (pid == 0) {
 	    m_indexerState = IXST_NOTRUNNING;
 	    fileToggleIndexingAction->setText(tr("Update &Index"));
 	    fileToggleIndexingAction->setEnabled(true);
@@ -173,15 +187,21 @@ void RclMain::toggleIndexing()
 	if (m_idxproc) {
 	    // Indexing was in progress, request stop. Let the periodic
 	    // routine check for the results.
-	    int pid = m_idxproc->getChildPid();
-	    if (pid > 0) {
-		kill(pid, SIGTERM);
+            if (m_idxproc->requestChildExit()) {
                 m_idxkilled = true;
             }
 	}
 	break;
     case IXST_RUNNINGNOTMINE:
     {
+#ifdef _WIN32
+        QMessageBox::warning(0, tr("Warning"),
+                             tr("The current indexing process "
+                                "was not started from this "
+                                "interface, can't kill it"),
+                             QMessageBox::Ok,
+                             QMessageBox::NoButton);
+#else
 	int rep = 
 	    QMessageBox::information(0, tr("Warning"), 
 				     tr("The current indexing process "
@@ -197,7 +217,8 @@ void RclMain::toggleIndexing()
 	    if (pid > 0)
 		kill(pid, SIGTERM);
 	}
-    }		
+#endif
+    }
     break;
     case IXST_NOTRUNNING:
     {
@@ -258,6 +279,13 @@ void RclMain::rebuildIndex()
 					 QMessageBox::Cancel,
 					 QMessageBox::NoButton);
 	if (rep == QMessageBox::Ok) {
+#ifdef _WIN32
+            // Under windows, it's necessary to close the db here, else Xapian
+            // won't be able to do what it wants with the (open) files. Of course
+            // if there are several GUI instances, this won't work...
+            if (rcldb)
+                rcldb->close();
+#endif // _WIN32
 	    // Could also mean that no helpers are missing, but then we
 	    // won't try to show a message anyway (which is what
 	    // firstIndexing is used for)
