@@ -30,6 +30,7 @@
 #include "fragbuts.h"
 #include "specialindex.h"
 #include "rclmain_w.h"
+#include "webcache.h"
 
 using namespace std;
 
@@ -48,8 +49,8 @@ void RclMain::showAdvSearchDialog()
 		this, SLOT (fileExit()));
 
 	connect(asearchform, 
-		SIGNAL(startSearch(RefCntr<Rcl::SearchData>, bool)), 
-		this, SLOT(startSearch(RefCntr<Rcl::SearchData>, bool)));
+		SIGNAL(startSearch(STD_SHARED_PTR<Rcl::SearchData>, bool)), 
+		this, SLOT(startSearch(STD_SHARED_PTR<Rcl::SearchData>, bool)));
 	asearchform->show();
     } else {
 	// Close and reopen, in hope that makes us visible...
@@ -74,6 +75,58 @@ void RclMain::showSpellDialog()
     }
 }
 
+void RclMain::showWebcacheDialog()
+{
+    switch (indexerState()) {
+    case RclMain::IXST_UNKNOWN:
+        QMessageBox::warning(0, "Recoll", tr("Unknown indexer state. "
+                                             "Can't access webcache file."));
+        return;
+    case RclMain::IXST_RUNNINGMINE:
+    case RclMain::IXST_RUNNINGNOTMINE:
+        QMessageBox::warning(0, "Recoll", tr("Indexer is running. "
+                                             "Can't access webcache file."));
+        return;
+    case RclMain::IXST_NOTRUNNING:
+        break;
+    }
+
+    if (!m_pidfile) {
+        m_pidfile = new Pidfile(theconfig->getPidfile());
+        if (m_pidfile->open() != 0) {
+            deleteZ(m_pidfile);
+            return;
+        }
+        if (m_pidfile->write_pid() != 0) {
+            deleteZ(m_pidfile);
+            return;
+        }
+    }
+    
+    if (webcache == 0) {
+	webcache = new WebcacheEdit(this);
+        webcache->setAttribute(Qt::WA_DeleteOnClose);
+        connect(new QShortcut(quitKeySeq, webcache), SIGNAL (activated()), 
+		this, SLOT (fileExit()));
+        connect(webcache, SIGNAL(destroyed(QObject*)),
+                this, SLOT(onWebcacheDestroyed(QObject*)) );
+        webcache->show();
+    }
+}
+void RclMain::onWebcacheDestroyed(QObject *)
+{
+    deleteZ(m_pidfile);
+    webcache = 0;
+}
+
+void RclMain::showIndexStatistics()
+{
+    showSpellDialog();
+    if (spellform == 0)
+        return;
+    spellform->setMode(SpellW::TYPECMB_STATS);
+}
+
 void RclMain::showFragButs()
 {
     if (fragbuts && fragbuts->isStale(0)) {
@@ -86,8 +139,7 @@ void RclMain::showFragButs()
             connect(fragbuts, SIGNAL(fragmentsChanged()),
                     this, SLOT(onFragmentsChanged()));
         } else {
-            delete fragbuts;
-            fragbuts = 0;
+            deleteZ(fragbuts);
         }
     } else {
 	// Close and reopen, in hope that makes us visible...
@@ -147,6 +199,12 @@ void RclMain::execIndexSched()
 }
 void RclMain::showIndexSched(bool modal)
 {
+#ifdef _WIN32
+    QMessageBox::information(this, tr("Index scheduling"),
+                             tr("Sorry, not available under Windows for now, use the File menu entries "
+                                "to update the index"));
+    return;
+#endif    
     LOGDEB(("showIndexSched()\n"));
     if (indexSched == 0) {
 	indexSched = new IdxSchedW(this);
@@ -252,17 +310,8 @@ void RclMain::showUIPrefs()
 
 void RclMain::showExtIdxDialog()
 {
-    if (uiprefs == 0) {
-	uiprefs = new UIPrefsDialog(this);
-	connect(new QShortcut(quitKeySeq, uiprefs), SIGNAL (activated()), 
-		this, SLOT (fileExit()));
-	connect(uiprefs, SIGNAL(uiprefsDone()), this, SLOT(setUIPrefs()));
-    } else {
-	// Close and reopen, in hope that makes us visible...
-	uiprefs->close();
-    }
+    showUIPrefs();
     uiprefs->tabWidget->setCurrentIndex(3);
-    uiprefs->show();
 }
 
 void RclMain::showAboutDialog()
