@@ -16,9 +16,9 @@
  */
 #include "autoconfig.h"
 
-#include <fcntl.h>
-
 #include <utility>
+#include MEMORY_INCLUDE
+#include <stdlib.h>
 
 #include <qapplication.h>
 #include <qmessagebox.h>
@@ -52,7 +52,6 @@
 #include "uiprefs_w.h"
 #include "guiutils.h"
 #include "reslist.h"
-#include "refcntr.h"
 #include "ssearch_w.h"
 #include "internfile.h"
 #include "docseqdb.h"
@@ -69,6 +68,20 @@
 #include "rclhelp.h"
 #include "moc_rclmain_w.cpp"
 
+/* Qt5 moc expands macros when defining signals. The SIGNAL() macro is
+   a stringification, so it does not expand macros. We have signals
+   where one of the types is a #define (for the variations on
+   std::shared_ptr). In qt5, the connection does not work because the
+   signal string is different between the definition and the connect
+   call, because of the different macro expansion. We have to use
+   another level of macro in Qt5 to force macro expansion, but not in
+   Qt4, so we both define the XSIGNAL and XSLOT macros here, and have
+   ifdefs in the code. What a mess... */
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+#define XSIGNAL(X) SIGNAL(X)
+#define XSLOT(X) SLOT(X)
+#endif
+
 using std::pair;
 
 QString g_stringAllStem, g_stringNoStem;
@@ -137,20 +150,28 @@ void RclMain::init()
 
     // idxstatus file. Make sure it exists before trying to watch it
     // (case where we're started on an older index, or if the status
-    // file was deleted since indexing
-    ::close(::open(theconfig->getIdxStatusFile().c_str(), O_CREAT, 0600));
-    m_watcher.addPath(QString::fromLocal8Bit(
-			  theconfig->getIdxStatusFile().c_str()));
+    // file was deleted since indexing)
+    QString idxfn = 
+        QString::fromLocal8Bit(theconfig->getIdxStatusFile().c_str());
+    QFile qf(idxfn);
+    qf.open(QIODevice::ReadWrite);
+    qf.setPermissions(QFile::ReadOwner|QFile::WriteOwner);
+    qf.close();
+    m_watcher.addPath(idxfn);
     // At least some versions of qt4 don't display the status bar if
     // it's not created here.
     (void)statusBar();
 
     (void)new HelpClient(this);
-    HelpClient::installMap((const char *)this->objectName().toUtf8(), "RCL.SEARCH.SIMPLE");
+    HelpClient::installMap((const char *)this->objectName().toUtf8(),
+                           "RCL.SEARCH.GUI.SIMPLE");
 
     // Set the focus to the search terms entry:
     sSearch->queryText->setFocus();
 
+    enbSynAction->setDisabled(prefs.synFile.isEmpty());
+    enbSynAction->setChecked(prefs.synFileEnable);
+    
     // Stemming language menu
     g_stringNoStem = tr("(no stemming)");
     g_stringAllStem = tr("(all languages)");
@@ -283,13 +304,20 @@ void RclMain::init()
     sc = new QShortcut(seql, this);
     connect(sc, SIGNAL (activated()), sSearch, SLOT (takeFocus()));
 
-    connect(&m_watcher, SIGNAL(fileChanged(QString)), 
-	    this, SLOT(idxStatus()));
-    connect(sSearch, SIGNAL(startSearch(RefCntr<Rcl::SearchData>, bool)), 
-	    this, SLOT(startSearch(RefCntr<Rcl::SearchData>, bool)));
+    connect(&m_watcher, SIGNAL(fileChanged(QString)),
+            this, SLOT(updateIdxStatus()));
+
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+    connect(sSearch,
+            XSIGNAL(startSearch(STD_SHARED_PTR<Rcl::SearchData>, bool)), 
+	    this, XSLOT(startSearch(STD_SHARED_PTR<Rcl::SearchData>, bool)));
+#else
+    connect(sSearch,
+            SIGNAL(startSearch(STD_SHARED_PTR<Rcl::SearchData>, bool)), 
+	    this, SLOT(startSearch(STD_SHARED_PTR<Rcl::SearchData>, bool)));
+#endif
     connect(sSearch, SIGNAL(clearSearch()), 
 	    this, SLOT(resetSearch()));
-
     connect(preferencesMenu, SIGNAL(triggered(QAction*)),
 	    this, SLOT(setStemLang(QAction*)));
     connect(preferencesMenu, SIGNAL(aboutToShow()),
@@ -308,7 +336,8 @@ void RclMain::init()
 	    this, SLOT(saveLastQuery()));
     connect(actionLoad_saved_query, SIGNAL(triggered()),
 	    this, SLOT(loadSavedQuery()));
-
+    connect(actionShow_index_statistics, SIGNAL(triggered()),
+            this, SLOT(showIndexStatistics()));
     connect(helpAbout_RecollAction, SIGNAL(triggered()), 
 	    this, SLOT(showAboutDialog()));
     connect(showMissingHelpers_Action, SIGNAL(triggered()), 
@@ -323,6 +352,12 @@ void RclMain::init()
 	    this, SLOT(showAdvSearchDialog()));
     connect(toolsSpellAction, SIGNAL(triggered()), 
 	    this, SLOT(showSpellDialog()));
+#ifdef _WIN32
+    actionWebcache_Editor->setEnabled(false);
+#else
+    connect(actionWebcache_Editor, SIGNAL(triggered()),
+            this, SLOT(showWebcacheDialog()));
+#endif
     connect(actionQuery_Fragments, SIGNAL(triggered()), 
 	    this, SLOT(showFragButs()));
     connect(actionSpecial_Indexing, SIGNAL(triggered()), 
@@ -335,6 +370,8 @@ void RclMain::init()
 	    this, SLOT(showUIPrefs()));
     connect(extIdxAction, SIGNAL(triggered()), 
 	    this, SLOT(showExtIdxDialog()));
+    connect(enbSynAction, SIGNAL(toggled(bool)),
+            this, SLOT(setSynEnabled(bool)));
 
     connect(toggleFullScreenAction, SIGNAL(triggered()), 
             this, SLOT(toggleFullScreen()));
@@ -346,8 +383,14 @@ void RclMain::init()
     restable->setRclMain(this, true);
     connect(actionSaveResultsAsCSV, SIGNAL(triggered()), 
 	    restable, SLOT(saveAsCSV()));
-    connect(this, SIGNAL(docSourceChanged(RefCntr<DocSequence>)),
-	    restable, SLOT(setDocSource(RefCntr<DocSequence>)));
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+    connect(this, XSIGNAL(docSourceChanged(STD_SHARED_PTR<DocSequence>)),
+	    restable, XSLOT(setDocSource(STD_SHARED_PTR<DocSequence>)));
+#else
+    connect(this, SIGNAL(docSourceChanged(STD_SHARED_PTR<DocSequence>)),
+	    restable, SLOT(setDocSource(STD_SHARED_PTR<DocSequence>)));
+#endif
+   
     connect(this, SIGNAL(searchReset()), 
 	    restable, SLOT(resetSource()));
     connect(this, SIGNAL(resultsReady()), 
@@ -376,8 +419,13 @@ void RclMain::init()
 	    this, SLOT(showSnippets(Rcl::Doc)));
 
     reslist->setRclMain(this, true);
-    connect(this, SIGNAL(docSourceChanged(RefCntr<DocSequence>)),
-	    reslist, SLOT(setDocSource(RefCntr<DocSequence>)));
+#if (QT_VERSION >= QT_VERSION_CHECK(5, 0, 0))
+    connect(this, XSIGNAL(docSourceChanged(STD_SHARED_PTR<DocSequence>)),
+	    reslist, XSLOT(setDocSource(STD_SHARED_PTR<DocSequence>)));
+#else
+    connect(this, SIGNAL(docSourceChanged(STD_SHARED_PTR<DocSequence>)),
+	    reslist, SLOT(setDocSource(STD_SHARED_PTR<DocSequence>)));
+#endif
     connect(firstPageAction, SIGNAL(triggered()), 
 	    reslist, SLOT(resultPageFirst()));
     connect(prevPageAction, SIGNAL(triggered()), 
@@ -442,6 +490,13 @@ void RclMain::init()
     periodictimer->start(1000);
 }
 
+void RclMain::setSynEnabled(bool on)
+{
+    prefs.synFileEnable = on;
+    if (uiprefs)
+        uiprefs->synFileCB->setChecked(prefs.synFileEnable);
+}
+
 void RclMain::resultCount(int n)
 {
     actionSortByDateAsc->setEnabled(n>0);
@@ -628,17 +683,19 @@ void RclMain::fileExit()
 	prefs.ssearchTyp = sSearch->searchTypCMB->currentIndex();
     }
 
-    if (asearchform)
-	delete asearchform;
-
     rwSettings(true);
 
-    // Let the exit handler clean up the rest (internal recoll stuff).
-    exit(0);
+    // We should do the right thing and let exit() call all the
+    // cleanup handlers. But we have few persistent resources and qt
+    // exit is a great source of crashes and pita. So do our own
+    // cleanup:
+    deleteAllTempFiles();
+    // and scram out
+    _Exit(0);
 }
 
 // Start a db query and set the reslist docsource
-void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata, bool issimple)
+void RclMain::startSearch(STD_SHARED_PTR<Rcl::SearchData> sdata, bool issimple)
 {
     LOGDEB(("RclMain::startSearch. Indexing %s Active %d\n", 
 	    m_idxproc?"on":"off", m_queryActive));
@@ -648,7 +705,7 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata, bool issimple)
     }
     m_queryActive = true;
     restable->setEnabled(false);
-    m_source = RefCntr<DocSequence>();
+    m_source = STD_SHARED_PTR<DocSequence>();
 
     m_searchIsSimple = issimple;
 
@@ -662,16 +719,27 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata, bool issimple)
 	return;
     }
 
+    if (prefs.synFileEnable && !prefs.synFile.isEmpty()) {
+        string sf = (const char *)prefs.synFile.toLocal8Bit();
+        if (!rcldb->setSynGroupsFile(sf)) {
+            QMessageBox::warning(0, "Recoll",
+                                 tr("Can't set synonyms file (parse error?)"));
+            return;
+        }
+    } else {
+        rcldb->setSynGroupsFile("");
+    }
+
     Rcl::Query *query = new Rcl::Query(rcldb);
     query->setCollapseDuplicates(prefs.collapseDuplicates);
 
     curPreview = 0;
     DocSequenceDb *src = 
-	new DocSequenceDb(RefCntr<Rcl::Query>(query), 
+	new DocSequenceDb(STD_SHARED_PTR<Rcl::Query>(query), 
 			  string(tr("Query results").toUtf8()), sdata);
     src->setAbstractParams(prefs.queryBuildAbstract, 
                            prefs.queryReplaceAbstract);
-    m_source = RefCntr<DocSequence>(src);
+    m_source = STD_SHARED_PTR<DocSequence>(src);
     m_source->setSortSpec(m_sortspec);
     m_source->setFiltSpec(m_filtspec);
 
@@ -682,9 +750,9 @@ void RclMain::startSearch(RefCntr<Rcl::SearchData> sdata, bool issimple)
 
 class QueryThread : public QThread {
     int loglevel;
-    RefCntr<DocSequence> m_source;
+    STD_SHARED_PTR<DocSequence> m_source;
  public: 
-    QueryThread(RefCntr<DocSequence> source)
+    QueryThread(STD_SHARED_PTR<DocSequence> source)
 	: m_source(source)
     {
 	loglevel = DebugLog::getdbl()->getlevel();
@@ -700,7 +768,7 @@ class QueryThread : public QThread {
 
 void RclMain::initiateQuery()
 {
-    if (m_source.isNull())
+    if (!m_source)
 	return;
 
     QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
@@ -729,7 +797,7 @@ void RclMain::initiateQuery()
 	qApp->processEvents();
 	if (progress.wasCanceled()) {
 	    // Just get out of there asap. 
-	    _exit(1);
+	    exit(1);
 	}
 
 	qApp->processEvents();
@@ -780,7 +848,7 @@ void RclMain::onSortCtlChanged()
 	prefs.sortActive = prefs.sortDesc = false;
 	prefs.sortField = "";
     }
-    if (m_source.isNotNull())
+    if (m_source)
 	m_source->setSortSpec(m_sortspec);
     emit sortDataChanged(m_sortspec);
     initiateQuery();
@@ -798,7 +866,7 @@ void RclMain::onSortDataChanged(DocSeqSortSpec spec)
 	actionSortByDateAsc->setChecked(!spec.desc);
     }
     m_sortspecnochange = false;
-    if (m_source.isNotNull())
+    if (m_source)
 	m_source->setSortSpec(spec);
     m_sortspec = spec;
 
@@ -898,8 +966,8 @@ void RclMain::showSubDocs(Rcl::Doc doc)
 	new DocSequenceDocs(rcldb, docs,
 			    qs2utf8s(tr("Sub-documents and attachments")));
     src->setDescription(qs2utf8s(tr("Sub-documents and attachments")));
-    RefCntr<DocSequence> 
-	source(new DocSource(theconfig, RefCntr<DocSequence>(src)));
+    STD_SHARED_PTR<DocSequence> 
+	source(new DocSource(theconfig, STD_SHARED_PTR<DocSequence>(src)));
 
     ResTable *res = new ResTable();
     res->setRclMain(this, false);
@@ -940,7 +1008,7 @@ void RclMain::showDocHistory()
 {
     LOGDEB(("RclMain::showDocHistory\n"));
     emit searchReset();
-    m_source = RefCntr<DocSequence>();
+    m_source = STD_SHARED_PTR<DocSequence>();
     curPreview = 0;
 
     string reason;
@@ -949,8 +1017,8 @@ void RclMain::showDocHistory()
 	return;
     }
     // Construct a bogus SearchData structure
-    RefCntr<Rcl::SearchData>searchdata = 
-	RefCntr<Rcl::SearchData>(new Rcl::SearchData(Rcl::SCLT_AND, cstr_null));
+    STD_SHARED_PTR<Rcl::SearchData>searchdata = 
+	STD_SHARED_PTR<Rcl::SearchData>(new Rcl::SearchData(Rcl::SCLT_AND, cstr_null));
     searchdata->setDescription((const char *)tr("History data").toUtf8());
 
 
@@ -959,8 +1027,8 @@ void RclMain::showDocHistory()
 	new DocSequenceHistory(rcldb, g_dynconf, 
 			       string(tr("Document history").toUtf8()));
     src->setDescription((const char *)tr("History data").toUtf8());
-    DocSource *source = new DocSource(theconfig, RefCntr<DocSequence>(src));
-    m_source = RefCntr<DocSequence>(source);
+    DocSource *source = new DocSource(theconfig, STD_SHARED_PTR<DocSequence>(src));
+    m_source = STD_SHARED_PTR<DocSequence>(source);
     m_source->setSortSpec(m_sortspec);
     m_source->setFiltSpec(m_filtspec);
     emit docSourceChanged(m_source);
@@ -997,6 +1065,8 @@ void RclMain::setUIPrefs()
     LOGDEB(("Recollmain::setUIPrefs\n"));
     reslist->setFont();
     sSearch->setPrefs();
+    enbSynAction->setDisabled(prefs.synFile.isEmpty());
+    enbSynAction->setChecked(prefs.synFileEnable);
 }
 
 void RclMain::enableNextPage(bool yesno)
@@ -1015,7 +1085,7 @@ void RclMain::enablePrevPage(bool yesno)
 
 QString RclMain::getQueryDescription()
 {
-    if (m_source.isNull())
+    if (!m_source)
 	return "";
     return QString::fromUtf8(m_source->getDescription().c_str());
 }
@@ -1075,7 +1145,7 @@ void RclMain::setFiltSpec()
         }
     }
 
-    if (m_source.isNotNull())
+    if (m_source)
 	m_source->setFiltSpec(m_filtspec);
     initiateQuery();
 }
