@@ -16,7 +16,7 @@
  */
 #include "autoconfig.h"
 
-#include <sys/stat.h>
+#include "safesysstat.h"
 
 #include <string>
 #include <algorithm>
@@ -45,7 +45,6 @@
 
 #include "recoll.h"
 #include "guiutils.h"
-#include "rcldb.h"
 #include "rclconfig.h"
 #include "pathut.h"
 #include "uiprefs_w.h"
@@ -73,6 +72,7 @@ void UIPrefsDialog::init()
     connect(stylesheetPB, SIGNAL(clicked()),this, SLOT(showStylesheetDialog()));
     connect(resetSSPB, SIGNAL(clicked()), this, SLOT(resetStylesheet()));
     connect(snipCssPB, SIGNAL(clicked()),this, SLOT(showSnipCssDialog()));
+    connect(synFilePB, SIGNAL(clicked()),this, SLOT(showSynFileDialog()));
     connect(resetSnipCssPB, SIGNAL(clicked()), this, SLOT(resetSnipCss()));
 
     connect(idxLV, SIGNAL(itemSelectionChanged()),
@@ -215,6 +215,15 @@ void UIPrefsDialog::setFromPrefs()
     autoSuffsCB->setChecked(prefs.autoSuffsEnable);
     autoSuffsLE->setText(prefs.autoSuffs);
 
+    synFileCB->setChecked(prefs.synFileEnable);
+    synFile = prefs.synFile;
+    if (synFile.isEmpty()) {
+	synFilePB->setText(tr("Choose"));
+    } else {
+	string nm = path_getsimple((const char *)synFile.toLocal8Bit());
+	synFilePB->setText(QString::fromLocal8Bit(nm.c_str()));
+    }
+
     // Initialize the extra indexes listboxes
     idxLV->clear();
     for (list<string>::iterator it = prefs.allExtraDbs.begin(); 
@@ -329,10 +338,12 @@ void UIPrefsDialog::accept()
     prefs.syntAbsLen = syntlenSB->value();
     prefs.syntAbsCtx = syntctxSB->value();
 
-    
     prefs.autoSuffsEnable = autoSuffsCB->isChecked();
     prefs.autoSuffs = autoSuffsLE->text();
 
+    prefs.synFileEnable = synFileCB->isChecked();
+    prefs.synFile = synFile;
+    
     prefs.allExtraDbs.clear();
     prefs.activeExtraDbs.clear();
     for (int i = 0; i < idxLV->count(); i++) {
@@ -445,6 +456,15 @@ void UIPrefsDialog::resetSnipCss()
     snipCssPB->setText(tr("Choose"));
 }
 
+void UIPrefsDialog::showSynFileDialog()
+{
+    synFile = myGetFileName(false, "Select synonyms file", true);
+    if (synFile.isEmpty())
+        return;
+    string nm = path_getsimple((const char *)synFile.toLocal8Bit());
+    synFilePB->setText(QString::fromLocal8Bit(nm.c_str()));
+}
+
 void UIPrefsDialog::resetReslistFont()
 {
     reslistFontFamily = "";
@@ -537,6 +557,9 @@ void UIPrefsDialog::delExtraDbPB_clicked()
 
 static bool samedir(const string& dir1, const string& dir2)
 {
+#ifdef _WIN32
+    return !dir1.compare(dir2);
+#else
     struct stat st1, st2;
     if (stat(dir1.c_str(), &st1))
 	return false;
@@ -546,6 +569,7 @@ static bool samedir(const string& dir1, const string& dir2)
 	return true;
     }
     return false;
+#endif
 }
 
 void UIPrefsDialog::on_showTrayIconCB_clicked()
@@ -569,7 +593,7 @@ void UIPrefsDialog::addExtraDbPB_clicked()
     if (input.isEmpty())
 	return;
     string dbdir = (const char *)input.toLocal8Bit();
-    if (access(path_cat(dbdir, "recoll.conf").c_str(), 0) == 0) {
+    if (path_exists(path_cat(dbdir, "recoll.conf"))) {
 	// Chosen dir is config dir.
 	RclConfig conf(&dbdir);
 	dbdir = conf.getDbDir();
